@@ -1,4 +1,5 @@
 from typing import Callable, Type
+from pydantic import BaseModel
 
 from scintillant.apimodels import SkillResponse
 from scintillant.apimodels.types import SkillRequestType, SkillResponseType
@@ -11,7 +12,9 @@ class ContextUpdater:
     states = {}  # state_name: state_func
     exit_phrases = ['exit', 'выход', 'хватит', 'остановись']
 
-    def __init__(self, data: SkillRequestType, dictionary: Type[Dictionary] = None):
+    def __init__(self, data: SkillRequestType,
+                 dictionary: Type[Dictionary] = None,
+                 vault: BaseModel = None):
         # Default parameters for find state and complete it
         self.data = data
         self.context = data.context
@@ -22,6 +25,12 @@ class ContextUpdater:
         elif dictionary:
             self.dictionary = dictionary.text()
 
+        if vault and '__vault__' in self.context:
+            self.vault = vault.parse_raw(self.context['__vault__'])
+        elif vault:
+            self.vault = vault
+            self.context['__vault__'] = self.vault.dict()
+
     @property
     def next_state(self):
         if 'state' in self.context:
@@ -31,7 +40,8 @@ class ContextUpdater:
     @next_state.setter
     def next_state(self, func: Callable):
         if func:
-            self.context['state'] = func.__qualname__.split('.')[0] + '_' + func.__name__
+            self.context['state'] = func.__qualname__.split('.')[
+                                        0] + '_' + func.__name__
         else:
             self.context.pop('state')
 
@@ -52,17 +62,27 @@ class ContextUpdater:
             else:
                 self.states[state](self)
 
+    def execute_now(self, func: Callable) -> None:
+        self.next_state = func
+        self.execute_state()
+
+    def redirect(self, redirect_to: str) -> None:
+        self.response.status = 'redirect'
+        self.response.redirect_to = redirect_to
+
     def get_response(self) -> SkillResponseType:
-        logger.debug("Executing state {} from User {} | Update data: {}".format(
-            self.next_state,
-            self.data.user,
-            self.data
-        ))
+        logger.debug(
+            "Executing state {} from User {} | Update data: {}".format(
+                self.next_state,
+                self.data.user,
+                self.data
+            ))
         self.execute_state()
         self.response.context = self.context
         logger.info("Getting response: {}".format(self.response))
         if not self.response.out_text and self.response.status not in ['redirect', 'exit']:
-            logger.error('Response out_text field is None inside non-action state!')
+            logger.error(
+                'Response out_text field is None inside non-action state!')
         return self.response
 
     def _initial_state_(self):
@@ -75,10 +95,16 @@ class ContextUpdater:
 
     @classmethod
     def statefunc(cls, func):
-        cls.states[func.__qualname__.split('.')[0] + '_' + func.__name__] = func
+        cls.states[
+            func.__qualname__.split('.')[0] + '_' + func.__name__] = func
 
         def wrapper(*args, **kwargs):
             func(*args, **kwargs)
-        wrapper.__name__ = func.__qualname__.split('.')[0] + '_' + func.__name__
+
+        wrapper.__name__ = func.__qualname__.split('.')[
+                               0] + '_' + func.__name__
 
         return wrapper
+
+    def setup_logs(self, folder, level='DEBUG', rotation='2 GB'):
+        logger.add(folder + f'{self.__name__}.log', level=level, rotation=rotation)
